@@ -28,12 +28,12 @@ import os.path
 LIBDIR = '/usr/lib'
 PKGDATADIR = '/usr/share/pitivi'
 PIXMAPDIR = '/usr/share/pixmaps'
-pitivi_version = '0.13.1'
+pitivi_version = '0.13.1.2'
 APPNAME = 'PiTiVi'
 PYGTK_REQ = '2.12.0'
 PYGST_REQ = '0.10.15'
-GST_REQ = '0.10.23'
-GNONLIN_REQ = '0.10.10.3'
+GST_REQ = '0.10.23.1'
+GNONLIN_REQ = '0.10.11.1'
 PYCAIRO_REQ = '1.0.0'
 
 def _get_root_dir():
