@@ -90,10 +90,7 @@ def beautify_stream(stream):
             templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
                     "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
                     stream.channels)
-            try:
-                templ = templ % (stream.channels, stream.rate, stream.width)
-            except TypeError:
-                import pdb; pdb.set_trace()
+            templ = templ % (stream.channels, stream.rate, stream.width)
             return templ
 
         return _("<b>Unknown Audio format:</b> %s") % stream.audiotype
@@ -122,7 +119,7 @@ def beautify_factory(factory):
 
     streams = factory.getOutputStreams()
     streams.sort(key=stream_sort_key)
-    return ("<b>" + escape(unquote(factory.displayname)) + "</b>\n" +
+    return ("<b>" + escape(unquote(factory.name)) + "</b>\n" +
         "\n".join((beautify_stream(stream) for stream in streams)))
 
 class SourceList(gtk.VBox, Loggable):
@@ -249,7 +246,8 @@ class SourceList(gtk.VBox, Loggable):
         # Connect to project.  We must remove and reset the callbacks when
         # changing project.
         self.project_signals = SignalGroup()
-        self._connectToProject(self.app.current)
+        self.app.connect("new-project-created",
+            self._newProjectCreatedCb)
         self.app.connect("new-project-loaded",
             self._newProjectLoadedCb)
         self.app.connect("new-project-failed",
@@ -300,6 +298,7 @@ class SourceList(gtk.VBox, Loggable):
 
         actiongroup = gtk.ActionGroup("sourcelistpermanent")
         actiongroup.add_actions(actions)
+        actiongroup.get_action("ImportSources").props.is_important = True
         uiman.insert_action_group(actiongroup, 0)
 
         self.selection_actions = gtk.ActionGroup("sourcelistselection")
@@ -318,14 +317,16 @@ class SourceList(gtk.VBox, Loggable):
         self._removeSources()
 
     def _insertEndCb(self, unused_action):
+        self.app.action_log.begin("add clip")
         timeline = self.app.current.timeline
         sources = self.app.current.sources
         start = timeline.duration
         for uri in self.getSelectedItems():
-            factory = sources[uri]
+            factory = sources.getUri(uri)
             source = timeline.addSourceFactory(factory)
             source.setStart(start)
             start += source.duration
+        self.app.action_log.commit()
 
     def _getIcon(self, iconname, alternate):
         icontheme = gtk.icon_theme_get_default()
@@ -348,13 +349,13 @@ class SourceList(gtk.VBox, Loggable):
 
         """
         self.project_signals.connect(
-            project.sources, "file_added", None, self._fileAddedCb)
+            project.sources, "source-added", None, self._sourceAddedCb)
         self.project_signals.connect(
-            project.sources, "file_removed", None, self._fileRemovedCb)
+            project.sources, "source-removed", None, self._sourceRemovedCb)
         self.project_signals.connect(
             project.sources, "discovery-error", None, self._discoveryErrorCb)
-        self.project_signals.connect(
-            project.sources, "missing-plugins", None, self._missingPluginsCb)
+        #self.project_signals.connect(
+        #    project.sources, "missing-plugins", None, self._missingPluginsCb)
         self.project_signals.connect(
             project.sources, "ready", None, self._sourcesStoppedImportingCb)
         self.project_signals.connect(
@@ -466,17 +467,17 @@ class SourceList(gtk.VBox, Loggable):
         self.storemodel.append([thumbnail,
             beautify_factory(factory),
             factory,
-            factory.name,
+            factory.uri,
             duration])
         self._displayTreeView()
 
     # sourcelist callbacks
 
-    def _fileAddedCb(self, unused_sourcelist, factory):
+    def _sourceAddedCb(self, unused_sourcelist, factory):
         """ a file was added to the sourcelist """
         self._addFactory(factory)
 
-    def _fileRemovedCb(self, unused_sourcelist, uri):
+    def _sourceRemovedCb(self, sourcelist, uri, factory):
         """ the given uri was removed from the sourcelist """
         # find the good line in the storemodel and remove it
         model = self.storemodel
@@ -551,7 +552,7 @@ class SourceList(gtk.VBox, Loggable):
         selected.sort(reverse=True)
         for path in selected:
             uri = model[path][COL_URI]
-            del self.app.current.sources[uri]
+            self.app.current.sources.removeUri(uri)
 
     ## UI Button callbacks
 
@@ -571,7 +572,7 @@ class SourceList(gtk.VBox, Loggable):
             return
         path = paths[0]
         factory = model[path][COL_FACTORY]
-        self.debug("Let's play %s", factory.name)
+        self.debug("Let's play %s", factory.uri)
         self.emit('play', factory)
 
     _dragStarted = False
@@ -654,19 +655,19 @@ class SourceList(gtk.VBox, Loggable):
         factory = self.storemodel[path][COL_FACTORY]
         self.emit('play', factory)
 
-    def _newProjectLoadedCb(self, unused_pitivi, project):
+    def _newProjectCreatedCb(self, app, project):
         # clear the storemodel
         self.storemodel.clear()
         self._connectToProject(project)
-        # synchronize the storemodel with the new project's sourcelist
-        if project.loaded:
-            for uri, factory in project.sources:
-                self.log("loading uri %s", uri)
-                self._addFactory(factory)
-        else:
-            if not self.infostub.showing:
-                self.pack_start(self.infostub, expand=False)
-                self.infostub.startingImport()
+
+
+    def _newProjectLoadingCb(self, unused_pitivi, uri):
+        if not self.infostub.showing:
+            self.pack_start(self.infostub, expand=False)
+            self.infostub.startingImport()
+
+    def _newProjectLoadedCb(self, unused_pitivi, project):
+        pass
 
     def _newProjectFailedCb(self, unused_pitivi, unused_reason,
         unused_uri):
@@ -744,8 +745,8 @@ class InfoStub(gtk.HBox, Loggable):
         self.errors = []
         self.showing = False
         self._importingmessage = _("Importing clips...")
-        self._errorsmessage = _("Error(s) occured while importing")
-        self._errormessage = _("An error occured while importing")
+        self._errorsmessage = _("Error(s) occurred while importing")
+        self._errormessage = _("An error occurred while importing")
         self._makeUI()
 
     def _makeUI(self):
