@@ -172,7 +172,10 @@ def data_probe(pad, data, section=""):
     else:
         log.debug("probe","%s EVENT %s", section, data.type)
         if data.type == gst.EVENT_NEWSEGMENT:
-            log.debug("probe","%s %r", section, list(data.parse_new_segment()))
+            upd, rat, fmt, start, stop, pos = data.parse_new_segment()
+            log.debug("probe","%s Update:%r rate:%f fmt:%s, start:%s, stop:%s, pos:%s",
+                      section, upd, rat, fmt, gst.TIME_ARGS(start),
+                      gst.TIME_ARGS(stop), gst.TIME_ARGS(pos))
     return True
 
 def linkDynamic(element, target):
@@ -231,25 +234,42 @@ def uri_is_reachable(uri):
     """
     if not uri_is_valid(uri):
         raise NotImplementedError(
+            # Translators: "non local" means the project is not stored
+            # on a local filesystem
             _("%s doesn't yet handle non local projects") % APPNAME)
     return os.path.isfile(gst.uri_get_location(uri))
 
 class PropertyChangeTracker(Signallable):
-    def __init__(self, timeline_object):
+    def __init__(self):
         self.properties = {}
+        self.obj = None
 
+    def connectToObject(self, obj):
+        self.obj = obj
+        self.properties = self._takeCurrentSnapshot(obj)
         for property_name in self.property_names:
-            self.properties[property_name] = \
-                    getattr(timeline_object, property_name)
-
-            timeline_object.connect(property_name + '-changed',
+            signal_name = property_name + '-changed'
+            self.__signals__[signal_name] = []
+            obj.connect(signal_name,
                     self._propertyChangedCb, property_name)
 
-    def _propertyChangedCb(self, timeline_object, value, property_name):
+    def _takeCurrentSnapshot(self, obj):
+        properties = {}
+        for property_name in self.property_names:
+            properties[property_name] = \
+                    getattr(obj, property_name.replace("-", "_"))
+
+        return properties
+
+    def disconnectFromObject(self, obj):
+        self.obj = None
+        obj.disconnect_by_func(self._propertyChangedCb)
+
+    def _propertyChangedCb(self, object, value, property_name):
         old_value = self.properties[property_name]
         self.properties[property_name] = value
 
-        self.emit(property_name + '-changed', timeline_object, old_value, value)
+        self.emit(property_name + '-changed', object, old_value, value)
 
 class Seeker(Signallable):
     __signals__ = {'seek': ['position', 'format']}
@@ -287,3 +307,134 @@ class Seeker(Signallable):
 
 def get_filesystem_encoding():
     return sys.getfilesystemencoding() or "utf-8"
+
+def get_controllable_properties(element):
+    """
+    Returns a list of controllable properties for the given
+    element (and child if it's a container).
+
+    The list is made of tuples containing:
+    * The GstObject
+    * The GParamspec
+    """
+    log.debug("utils", "element %r, %d", element, isinstance(element, gst.Bin))
+    res = []
+    if isinstance(element, gst.Bin):
+        for child in element.elements():
+            res.extend(get_controllable_properties(child))
+    else:
+        for prop in gobject.list_properties(element):
+            if prop.flags & gst.PARAM_CONTROLLABLE:
+                log.debug("utils", "adding property %r", prop)
+                res.append((element, prop))
+    return res
+
+def start_insort_left(a, x, lo=0, hi=None):
+    if hi is None:
+        hi = len(a)
+    while lo < hi:
+        mid = (lo+hi)//2
+        if a[mid].start < x.start: lo = mid+1
+        else: hi = mid
+    a.insert(lo, x)
+
+def start_insort_right(a, x, lo=0, hi=None):
+    if hi is None:
+        hi = len(a)
+    while lo < hi:
+        mid = (lo+hi)//2
+        if x.start < a[mid].start: hi = mid
+        else: lo = mid+1
+    a.insert(lo, x)
+
+def start_bisect_left(a, x, lo=0, hi=None):
+    if hi is None:
+        hi = len(a)
+    while lo < hi:
+        mid = (lo+hi)//2
+        if a[mid].start < x.start: lo = mid+1
+        else: hi = mid
+    return lo
+
+class Infinity(object):
+    def __cmp__(self, other):
+        if isinstance(other, Infinity):
+            return 0
+
+        return 1
+
+infinity = Infinity()
+
+def findObject(obj, objects):
+    low = 0
+    high = len(objects)
+    while low < high:
+        low = start_bisect_left(objects, obj, lo=low)
+        if low == high:
+            break
+
+        if objects[low] is obj:
+            return low
+        else:
+            low = low + 1
+
+    return low
+
+def getPreviousObject(obj, objects, priority=-1, exclude=None):
+    if priority == -1:
+        priority = obj.priority
+
+    if exclude is None:
+        exclude = []
+
+    obj_index = findObject(obj, objects)
+    if obj_index is None:
+        import pdb; pdb.set_trace()
+    # check if there are same-start objects
+    prev_obj_index = obj_index + 1
+    while prev_obj_index < len(objects):
+        prev_obj = objects[prev_obj_index]
+        if prev_obj in exclude:
+            continue
+
+        if prev_obj.start != obj.start:
+            break
+
+        if priority is None or prev_obj.priority == priority:
+            return prev_obj
+
+        prev_obj_index += 1
+
+    # check if there are objects with start < obj.start
+    prev_obj_index = obj_index - 1
+    while prev_obj_index >= 0:
+        prev_obj = objects[prev_obj_index]
+        if prev_obj not in exclude and \
+                (priority is None or \
+                    prev_obj.priority == priority):
+            return prev_obj
+
+        prev_obj_index -= 1
+
+    return None
+
+def getNextObject(obj, objects, priority=-1, exclude=None):
+    if priority == -1:
+        priority = obj.priority
+
+    if exclude is None:
+        exclude = []
+
+    obj_index = findObject(obj, objects)
+    next_obj_index = obj_index + 1
+    objs_len = len(objects)
+    while next_obj_index < objs_len:
+        next_obj = objects[next_obj_index]
+        if next_obj not in exclude and \
+                (priority is None or \
+                    next_obj.priority == priority):
+            return next_obj
+
+        next_obj_index += 1
+
+    return None
